import polars as pl

from lakefs_spec import LakeFSFileSystem

fs = LakeFSFileSystem()

with fs.transaction("quickstart", "main") as tx:
    lakes = pl.read_parquet(f"lakefs://quickstart/{tx.branch.id}/lakes.parquet")
    us_lakes = lakes.filter(pl.col("Country") == "United States of America")

    with fs.open(f"lakefs://quickstart/{tx.branch.id}/us_lakes.csv", "wb") as f:
        us_lakes.write_csv(f)  # (1)!

    tx.commit(message="Add US lakes")



from typing import Any

import pytest
from lakefs.branch import Branch
from lakefs.repository import Repository

from lakefs_spec import LakeFSFileSystem
from tests.util import RandomFileFactory


def test_transaction_commit(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    random_file = random_file_factory.make()

    lpath = str(random_file)
    rpath = f"{repository.id}/{temp_branch.id}/{random_file.name}"

    message = f"Add file {random_file.name}"

    with fs.transaction(repository, temp_branch) as tx:
        fs.put_file(lpath, f"{repository.id}/{tx.branch.id}/{random_file.name}")
        assert len(tx.files) == 1
        # sha is a placeholder for the actual SHA created on transaction completion.
        sha = tx.commit(message=message)

    # HEAD should be the merge commit.
    head_tilde = list(temp_branch.log(max_amount=2))[-1]
    assert head_tilde.message == message
    assert head_tilde.id == sha.id


def test_transaction_tag(fs: LakeFSFileSystem, repository: Repository) -> None:
    try:
        # tag gets created on exit of the context.
        # in this test, initialize with the repo name.
        with fs.transaction(repository.id) as tx:
            sha = tx.rev_parse("main")
            tag = tx.tag(sha, "v2")

        tags = list(repository.tags())
        assert len(tags) > 0
        assert tags[0].id == tag.id
        assert tags[0].get_commit().id == sha.id
    finally:
        tag.delete()


def test_transaction_merge(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
    temporary_branch_context: Any,
) -> None:
    random_file = random_file_factory.make()

    with temporary_branch_context("transaction-merge-test") as new_branch:
        message = "Commit new file"

        with fs.transaction(repository, new_branch) as tx:
            tbname = tx.branch.id
            lpath = str(random_file)
            # stage a file on the transaction branch...
            fs.put_file(lpath, f"{repository.id}/{tx.branch.id}/{random_file.name}")
            # ... commit it with the above message
            tx.commit(message=message)
            # ... and merge it into temp_branch.
            tx.merge(tx.branch, into=temp_branch)

        head, head_tilde = list(temp_branch.log(max_amount=2))
        # HEAD should be the merge commit of the transaction branch.
        assert head.message.startswith(f"Merge {tbname!r}")
        # HEAD~ should be the commit message.
        assert head_tilde.message == message


def test_transaction_revert(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    random_file = random_file_factory.make()

    lpath = str(random_file)
    message = f"Add file {random_file.name}"

    with fs.transaction(repository, temp_branch, automerge=True) as tx:
        fs.put_file(lpath, f"{repository.id}/{tx.branch.id}/{random_file.name}")
        tx.commit(message=message)
        revert_commit = tx.revert(temp_branch, temp_branch.head)

    # first commit should be the merge commit
    assert temp_branch.head.get_commit().message.startswith("Merge")
    assert revert_commit.message.startswith("Revert")


def test_transaction_failure(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    random_file = random_file_factory.make()

    lpath = str(random_file)
    rpath = f"{repository.id}/{temp_branch.id}/{random_file.name}"
    message = f"Add file {random_file.name}"

    try:
        with fs.transaction(repository, temp_branch) as tx:
            fs.put_file(lpath, f"{repository.id}/{tx.branch.id}/{random_file.name}")
            tx.commit(message=message)
            raise RuntimeError("something went wrong")
    except RuntimeError:
        pass

    # assert that no commit happens because of the exception.
    assert not fs.exists(rpath)


def test_transaction_no_automerge(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    currhead = temp_branch.head.get_commit()

    with fs.transaction(repository, temp_branch, automerge=False, delete="never") as tx:
        transaction_branch = tx.branch

    try:
        # assert no merge commit is created on temp_branch.
        assert currhead == next(temp_branch.log())
        # assert the transaction branch still exists.
        assert transaction_branch.id in [b.id for b in repository.branches()]
    finally:
        transaction_branch.delete()


def test_transaction_bad_repo(fs: LakeFSFileSystem) -> None:
    with pytest.raises(ValueError, match="repository .* does not exist"):
        with fs.transaction(repository="REEEE"):
            pass


def test_warn_uncommitted_changes(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    random_file = random_file_factory.make()

    lpath = str(random_file)

    with pytest.warns(match="uncommitted changes.*lost"):
        with fs.transaction(repository, temp_branch) as tx:
            fs.put_file(lpath, f"{repository.id}/{tx.branch.id}/{random_file.name}")


def test_warn_uncommitted_changes_on_persisted_branch(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    random_file = random_file_factory.make()

    lpath = str(random_file)

    with pytest.warns(match="uncommitted changes(?:(?!lost).)*$"):
        with fs.transaction(repository, temp_branch, delete="never") as tx:
            fs.put_file(lpath, f"{repository.id}/{tx.branch.id}/{random_file.name}")



import contextlib
import logging
import random
import string
import sys
from collections.abc import Generator
from pathlib import Path
from typing import Any, TypeVar
from unittest.mock import MagicMock

import lakefs
import pytest
import yaml
from lakefs.client import Client
from lakefs.repository import Repository

from lakefs_spec import LakeFSFileSystem
from tests.util import RandomFileFactory

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler(stream=sys.stdout))

T = TypeVar("T")

YieldFixture = Generator[T, None, None]


def pytest_report_header(config):
    from importlib.metadata import version

    lakefs_version = version("lakefs")
    from lakefs_sdk import __version__ as __lakefs_sdk_version__

    from lakefs_spec import __version__ as __lakefs_spec_version__

    return [
        f"lakefs version: {lakefs_version}",
        f"lakeFS SDK version: {__lakefs_sdk_version__}",
        f"lakeFS-spec version: {__lakefs_spec_version__}",
    ]


@pytest.fixture
def fs() -> LakeFSFileSystem:
    LakeFSFileSystem.clear_instance_cache()
    return LakeFSFileSystem(
        host="localhost:8000",
        username="AKIAIOSFOLQUICKSTART",
        password="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY",
    )


@pytest.fixture(scope="session")
def _client() -> Client:
    """A lakeFS client for API operations outside of the file system."""
    return Client(
        host="localhost:8000",
        username="AKIAIOSFOLQUICKSTART",
        password="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY",
    )


@pytest.fixture(scope="session")
def repository(_client: Client) -> Repository:
    name = "lakefs-spec-tests"
    storage_namespace = f"{_client.storage_config.default_namespace_prefix}/{name}"
    repo = lakefs.Repository(name, client=_client).create(
        storage_namespace=storage_namespace, include_samples=True, exist_ok=True
    )
    return repo


@pytest.fixture
def temporary_branch_context(repository: Repository) -> Any:
    @contextlib.contextmanager
    def _wrapper(name: str) -> YieldFixture[str]:
        branch = repository.branch(name)
        try:
            yield branch.create("main", exist_ok=False)
        finally:
            branch.delete()

    return _wrapper


@pytest.fixture
def temp_branch(repository: str, temporary_branch_context: Any) -> YieldFixture[str]:
    """Create a temporary branch for a test."""
    name = "test-" + "".join(random.choices(string.digits, k=8))
    with temporary_branch_context(name) as tb:
        yield tb


@pytest.fixture
def random_file_factory(tmp_path: Path) -> RandomFileFactory:
    return RandomFileFactory(path=tmp_path)


@pytest.fixture
def temporary_lakectl_config() -> YieldFixture[str]:
    d = {
        "credentials": {"access_key_id": "hello", "secret_access_key": "world"},
        "server": {"endpoint_url": "http://hello-world-xyz"},
    }

    loc = "~/.lakectl.yaml"
    path = Path(loc).expanduser()
    backup_path = path.with_stem(path.stem + "_BAK")

    try:
        if path.exists():
            path.rename(backup_path)
        with open(path, "w") as f:
            yaml.dump(d, f)
        yield loc
    finally:
        path.unlink()
        if backup_path.exists():
            backup_path.rename(path)


@pytest.fixture()
def mock_urlopen(monkeypatch: pytest.MonkeyPatch) -> MagicMock:
    mock = MagicMock()
    monkeypatch.setattr("urllib.request.urlopen", mock)
    return mock



import json

from lakefs.exceptions import ServerException

from lakefs_spec.errors import translate_lakefs_error


def test_error_translation() -> None:
    rpath = "repo/ref/ohno.txt"

    # first case: lakeFS API error, 401 unauthorized
    e = ServerException(
        status=401, reason="unauthorized", body=json.dumps({"message": "unauthorized"})
    )

    translated_err = translate_lakefs_error(e, rpath=rpath)
    assert isinstance(translated_err, PermissionError)
    assert f"unauthorized: {rpath!r}" in str(translated_err)

    # second case: lakeFS API error 420 (corresponds to partial IOError)
    e = ServerException(
        status=420, reason="too many requests", body=json.dumps({"message": "too many requests"})
    )
    translated_err = translate_lakefs_error(e, rpath=rpath)
    assert isinstance(translated_err, OSError)
    assert f"too many requests: {rpath!r}" in str(translated_err)

    # third case: lakeFS API error 400 with a custom message.
    e = ServerException(
        status=400, reason="bad request", body=json.dumps({"message": "bad request"})
    )
    message = "oh no!"
    translated_err = translate_lakefs_error(e, message=message)
    assert isinstance(translated_err, OSError)
    assert str(translated_err).endswith(message)



from lakefs.branch import Branch
from lakefs.repository import Repository

from lakefs_spec import LakeFSFileSystem
from tests.util import RandomFileFactory


def test_rm(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    path = f"{repository.id}/{temp_branch.id}/README.md"

    fs.rm(path)
    assert not fs.exists(path)


def test_rm_with_transaction(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    path = f"{repository.id}/{temp_branch.id}/README.md"
    message = "Remove file README.md"

    with fs.transaction(repository, temp_branch, automerge=True) as tx:
        fs.rm(f"{repository.id}/{tx.branch.id}/README.md")
        tx.commit(message=message)

    commits = list(temp_branch.log(max_amount=2))
    assert not fs.exists(path)
    assert commits[-1].message == message
    assert commits[0].message.startswith("Merge")


def test_rm_recursive(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    """Validate that recursive ``rm`` removes subdirectories as well."""
    prefix = f"lakefs://{repository.id}/{temp_branch.id}"

    fs.pipe(f"{prefix}/dir1/b.txt", b"b")
    fs.pipe(f"{prefix}/dir1/dir2/c.txt", b"c")

    fs.rm(f"{prefix}/dir1", recursive=False)
    assert fs.exists(f"{prefix}/dir1/dir2/c.txt")
    fs.rm(f"{prefix}/dir1", recursive=True)
    assert not fs.exists(f"{prefix}/dir1/dir2/c.txt")


def test_rm_recursive_with_maxdepth(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    """
    Check that recursive ``rm`` with maxdepth leaves directories beyond maxdepth untouched.
    """
    prefix = f"lakefs://{repository.id}/{temp_branch.id}"

    fs.pipe(f"{prefix}/dir1/b.txt", b"b")
    fs.pipe(f"{prefix}/dir1/dir2/c.txt", b"c")

    fs.rm(f"{prefix}/dir1", recursive=True, maxdepth=1)
    # maxdepth is 1-indexed, level 1 being the directory to be removed.
    assert fs.exists(f"{prefix}/dir1/dir2/c.txt")


def test_rm_with_1k_objects_or_more(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
    random_file_factory: RandomFileFactory,
) -> None:
    """
    Confirm that lakeFS does not error when attempting to delete more than 1k objects.
    """
    testdir = f"{repository.id}/{temp_branch.id}/subfolder"

    # create and put 1001 objects into the above lakeFS directory.
    for i in range(1002):
        f = random_file_factory.make()
        lpath = str(f)
        rpath = testdir + f"/test_{i}.txt"
        fs.put_file(lpath, rpath)

    assert len(fs.ls(testdir, detail=False)) > 1000

    # should not error, because we chunk the file deletion requests to size 1000.
    fs.rm(testdir, recursive=True, maxdepth=1)



"""Automatically convert Jupytext files to Notebooks"""

import logging
from pathlib import Path

import jupytext
import mkdocs_gen_files

for path in sorted(Path("docs/tutorials").rglob("*.py")):
    ipynb_path = path.relative_to(Path("docs")).with_suffix(".ipynb")
    logging.debug(f"Converting {path} -> {ipynb_path}")

    jupytext_file = path.read_text(encoding="utf-8")
    jupytext_nb = jupytext.reads(jupytext_file, fmt="py:percent")

    with mkdocs_gen_files.open(ipynb_path, "wb") as fd:
        content = jupytext.writes(jupytext_nb, fmt="notebook")
        fd.write(content.encode("utf8"))

    mkdocs_gen_files.set_edit_path(ipynb_path, path)



from pytest import MonkeyPatch

from lakefs_spec import LakeFSFileSystem


def test_instance_caching(fs: LakeFSFileSystem) -> None:
    assert len(LakeFSFileSystem._cache) == 1
    # same as the fixture, so we should get a cache hit.
    fs_new = LakeFSFileSystem(
        host="localhost:8000",
        username="AKIAIOSFOLQUICKSTART",
        password="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY",
    )

    assert fs_new._fs_token in LakeFSFileSystem._cache
    assert fs_new._fs_token == fs._fs_token

    # this, however, should not be cached.
    fs2 = LakeFSFileSystem(
        host="localhost:80000",
    )

    assert fs2._fs_token != fs._fs_token
    assert len(LakeFSFileSystem._cache) == 2


def test_initialization(monkeypatch: MonkeyPatch, temporary_lakectl_config: str) -> None:
    """
    Verify the priority order of initialization in lakeFS file system:
    1. direct arguments
    2. environment variables
    3. `lakectl` config file

    NOTE: The configuration appends "/api/v1" to all hostnames by default.
    """

    # Verify behaviors in reverse priority order.
    # Case 1: Instantiation from ~/.lakectl.yaml.
    cfg_fs = LakeFSFileSystem()
    config = cfg_fs.client.config
    assert config.host == "http://hello-world-xyz/api/v1"
    assert config.username == "hello"
    assert config.password == "world"

    monkeypatch.setenv("LAKECTL_SERVER_ENDPOINT_URL", "http://localhost:1000")
    monkeypatch.setenv("LAKECTL_CREDENTIALS_ACCESS_KEY_ID", "my-user")
    monkeypatch.setenv("LAKECTL_CREDENTIALS_SECRET_ACCESS_KEY", "my-password-12345")

    # Case 2: Instantiation from envvars.
    # Clear the instance cache first, since we otherwise would get a cache hit
    # due to the instantiation being the same as from `.lakectl.yaml` above.
    LakeFSFileSystem.clear_instance_cache()
    envvar_fs = LakeFSFileSystem()
    config = envvar_fs.client.config
    assert config.host == "http://localhost:1000/api/v1"
    assert config.username == "my-user"
    assert config.password == "my-password-12345"

    # Case 3: Explicit initialization.
    fs = LakeFSFileSystem(host="http://lakefs.hello", username="my-user", password="my-password")
    config = fs.client.config
    assert config.host == "http://lakefs.hello/api/v1"
    assert config.username == "my-user"
    assert config.password == "my-password"



from datasets import load_dataset

from lakefs_spec import LakeFSFileSystem

fs = LakeFSFileSystem()

with fs.transaction("quickstart", "main") as tx:
    lakes = load_dataset("parquet", data_files="lakefs://quickstart/main/lakes.parquet")
    irish_lakes = lakes.filter(lambda lake: lake["Country"] == "Ireland")
    irish_lakes.save_to_disk(f"lakefs://quickstart/{tx.branch.id}/irish_lakes")

    tx.commit(message="Add Irish lakes dataset")



from pathlib import Path

import pytest
from lakefs.branch import Branch
from lakefs.repository import Repository

from lakefs_spec import LakeFSFileSystem
from tests.util import RandomFileFactory, put_random_file_on_branch


def test_lakefs_file_open_read(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
    random_file_factory: RandomFileFactory,
) -> None:
    rpath = put_random_file_on_branch(random_file_factory, fs, repository, temp_branch)
    lpath = str(random_file_factory.path / Path(rpath).name)

    with open(lpath, "rb") as f:
        orig_text = f.read()

    # try opening the remote file
    with fs.open(rpath) as fp:
        text = fp.read()

    assert text == orig_text


def test_lakefs_file_open_write(
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
    random_file_factory: RandomFileFactory,
) -> None:
    rpath = put_random_file_on_branch(random_file_factory, fs, repository, temp_branch)
    lpath = str(random_file_factory.path / Path(rpath).name)

    with open(lpath, "rb") as f:
        orig_text = f.read()

    # try opening the remote file and writing to it
    with fs.open(rpath, "wb") as fp:
        fp.write(orig_text)

    # pulling the written file down again, using ONLY built-in open (!)
    plpath = Path(lpath)
    lpath = plpath.with_name(plpath.name + "_copy")

    blocksize = fs.blocksize
    fs.blocksize = 256
    fs.get(rpath, str(lpath))
    fs.blocksize = blocksize

    with open(lpath, "rb") as f:
        new_text = f.read()

    # round-trip assert.
    assert new_text == orig_text


def test_open_mode_coercion(fs: LakeFSFileSystem, repository: Repository) -> None:
    """Checks that text mode indicators are stripped."""
    with fs.open(f"{repository.id}/main/README.md", "rt") as f:
        assert f.mode == "r"


def test_lakefs_file_unknown_mode(fs: LakeFSFileSystem) -> None:
    """Test that a NotImplementedError is raised on unknown mode encounter."""

    with pytest.raises(NotImplementedError, match="unsupported mode .*"):
        fs.open("hello.py", mode="ab")  # type: ignore



import lakefs
from lakefs.branch import Branch
from lakefs.repository import Repository

from lakefs_spec import LakeFSFileSystem
from tests.util import RandomFileFactory


def test_exists(fs: LakeFSFileSystem, repository: Repository) -> None:
    """Test `fs.exists` on an existing file. Requires a populated repository."""

    example_file = f"{repository.id}/main/README.md"
    assert fs.exists(example_file)

    nonexistent_file = f"{repository.id}/main/nonexistent.parquet"
    assert not fs.exists(nonexistent_file)


def test_exists_on_commit(fs: LakeFSFileSystem, repository: Repository) -> None:
    """Test `fs.exists` works on commit SHAs to query existence of files in revisions."""
    example_file = f"{repository.id}/main/README.md"
    assert fs.exists(example_file)
    head = lakefs.Branch(repository.id, "main", client=fs.client).head
    assert fs.exists(f"{repository.id}/{head.id}/README.md")


def test_exists_on_staged_file(
    random_file_factory: RandomFileFactory,
    fs: LakeFSFileSystem,
    repository: Repository,
    temp_branch: Branch,
) -> None:
    random_file = random_file_factory.make()

    lpath = str(random_file)
    rpath = f"{repository.id}/{temp_branch.id}/{random_file.name}"

    # upload, verify existence.
    fs.put(lpath, rpath)
    assert fs.exists(rpath)


def test_exists_repo_root(
    fs: LakeFSFileSystem,
    repository: Repository,
) -> None:
    """Test `fs.exists` on the repository root."""

    # Existing repo and branch should return true
    assert fs.exists(f"lakefs://{repository.id}/main/")

    # Existing repo and commit should return true
    head_ref = lakefs.Branch(repository.id, "main", client=fs.client).head
    assert fs.exists(f"lakefs://{repository.id}/{head_ref.id}/")

    # Nonexistent branch should return false
    assert not fs.exists(f"lakefs://{repository.id}/nonexistent/")

    # Nonexistent repo should return false
    assert not fs.exists("lakefs://nonexistent/main/")



"""Automatically generate API reference pages from source files.

Source: https://mkdocstrings.github.io/recipes/#automatic-code-reference-pages

Note: this script assumes a source layout with a `src/` folder.
"""

import ast
import logging
from pathlib import Path

import docstring_parser
import mkdocs_gen_files

nav = mkdocs_gen_files.Nav()

for path in sorted(Path("src").rglob("*.py")):
    module_path = path.relative_to("src").with_suffix("")
    doc_path = path.relative_to("src").with_suffix(".md")
    full_doc_path = Path("reference", doc_path)

    parts = list(module_path.parts)

    if parts[-1] == "__init__":
        parts = parts[:-1]
        doc_path = doc_path.with_name("index.md")
        full_doc_path = full_doc_path.with_name("index.md")
    elif parts[-1] == "__main__":
        continue

    nav[parts] = doc_path.as_posix()

    with mkdocs_gen_files.open(full_doc_path, "w") as fd:
        identifier = ".".join(parts)
        print("::: " + identifier, file=fd)

    mkdocs_gen_files.set_edit_path(full_doc_path, path)

# Add links for top-level modules to root page
root_page = next(it for it in nav.items() if it.level == 0)
children = [it for it in nav.items() if it.level == 1]

with mkdocs_gen_files.open(f"reference/{root_page.filename}", "a") as f:
    f.write("## Modules\n")
    for ch in children:
        f.write(f"### [{ch.title}](../{ch.filename})\n")

        try:
            source_file = Path("src", ch.filename).with_suffix(".py")

            # Index page for submodules maps to __init__.py of the module
            if source_file.stem == "index":
                source_file = source_file.with_stem("__init__")

            tree = ast.parse(source_file.read_text())
            docstring = ast.get_docstring(tree, clean=False)
            doc = docstring_parser.parse(docstring)

            if doc.short_description:
                f.write(f"{doc.short_description}\n\n")
        except Exception as e:
            logging.warning(f"Could not parse module docstring: {ch.filename}", exc_info=True)

with mkdocs_gen_files.open("reference/SUMMARY.md", "w") as nav_file:
    nav_file.writelines(nav.build_literate_nav())



import duckdb
import numpy as np
import pandas as pd
import polars as pl
from lakefs.branch import Branch
from lakefs.repository import Repository

from lakefs_spec.spec import LakeFSFileSystem

storage_options = dict(
    host="localhost:8000",
    username="AKIAIOSFOLQUICKSTART",
    password="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY",
)


def test_pandas_integration(
    fs: LakeFSFileSystem, repository: Repository, temp_branch: Branch
) -> None:
    """Assure the correctness of pandas DataFrame reads and writes, which use `fs.open()`."""
    df = pd.read_parquet(
        f"lakefs://{repository.id}/{temp_branch.id}/lakes.parquet", storage_options=storage_options
    )
    df["randomcol"] = np.random.randn(len(df.index))
    df.to_parquet(
        f"lakefs://{repository.id}/{temp_branch.id}/lakes_new.parquet",
        storage_options=storage_options,
    )
    assert fs.exists(f"lakefs://{repository.id}/{temp_branch.id}/lakes_new.parquet")


def test_polars_integration(repository: Repository) -> None:
    """Test the download and instantiation of polars DataFrames via `fs.open()`."""
    pl.read_parquet(
        f"lakefs://{repository.id}/main/lakes.parquet",
        use_pyarrow=True,
        storage_options=storage_options,
    )


def test_duckdb_integration(fs: LakeFSFileSystem, repository: Repository) -> None:
    """Test the correct registration of the lakeFS file system in duckDB."""
    # see https://duckdb.org/docs/guides/python/filesystems.html
    duckdb.register_filesystem(fs)
    duckdb.read_parquet(f"lakefs://{repository.id}/main/lakes.parquet")
